badpvp = badpvp or {}

function badpvp.Setup()
    badpvp.classes = {}

    -- To add a new class, put the name here
    badpvp.classes.implemented = {
        "Psion",
        "Alchemist"
    }

    -- Then in here,  set the class name from above equal to the main object in that classes pvp script
    -- Important Note: You always need a name.decideAction function as this is called to generate pvp actions
    badpvp.classes.implementations = {
        Psion = psion,
        Alchemist = alchemist
    }

    -- Classes where we want to push stupidity
    badpvp.classes.stupidity = {
        "Alchemist"
    }

    -- Classes where we want to push clumsiness
    badpvp.classes.clumsiness = {
        "Blademaster",
        "Druid",
        "Infernal",
        "Monk",
        "Paladin",
        "Runewarden",
        "Priest",
        "Sentinel",
        "Serpent",
        "Shaman",
        "Psion",
        "Depthswalker"
    }

    -- classes where we want to push weariness
    badpvp.classes.weariness = {
        "Blademaster",
        "Druid",
        "Monk",
        "Priest",
        "Pariah",
        "Occultist",
        "Sentinel",
        "Serpent"
    }

    -- classes where we want to push confusion
    badpvp.classes.confusion = {
        "Psion"
    }

    -- classes where we want to push haemophilia
    badpvp.classes.haemophilia = {
        "Magi"
    }

    -- Don't change this section
    badpvp.groupsSetup()
    -- End Don't change

    -- Here is an example of adding additional setup for your pvp systems
    -- This will set the variable alchemist.specialty to Sublimation or Formulation depending
    alchemist.setSpecialty()
end

registerAnonymousEventHandler("sysLoadEvent", "badpvp.Setup")

function badpvp.groupsSetup()
    badpvp.groups = {}

    badpvp.groups.overrideAllies = false

    badpvp.groups.leaders = {
        "Bulwer",
        "Nezaya",
        "Belaziel"
    }

    badpvp.groups.trustedCity = "Hashan"

    badpvp.groups.allies = {
        "Cyrene"
    }

    badpvp.groups.alwaysTarget = {
        "Archaeon"
    }

    badpvp.groups.targetList = {}

    badpvp.groups.targetIndex = 1

    badpvp.groups.deadTargets = {}

    badpvp.groups.targetVisible = false
end

function badpvp.verifyInstall(_, name)
    if name ~= "BadPVP" then return end
    badpvp.Setup()
end

registerAnonymousEventHandler("sysInstall", "badpvp.verifyInstall")

function badpvp.verifyUninstall(_, name)
    if name ~= "BadPVP" then return end
    badpvp = nil
    alchemist = nil
    psion = nil
end
registerAnonymousEventHandler("sysUninstall", "badpvp.verifyUninstall")

function badpvp.verifyTarget(name)
    if table.contains(Legacy.NDB.db, name) then
        if table.contains(badpvp.groups.alwaysTarget, name) or table.contains(Legacy.CT.Enemies, name) then
            -- Always target enemies or those listed in the override table
            return true
        elseif Legacy.NDB.db[name].city == badpvp.groups.trustedCity then
            -- Always ignore targetting anyone from Hashan
            cecho("\n<firebrick>COMBAT NOTICE: Ignoring target call for " .. name .. " as they are Hashani.")
            return false
        elseif table.contains(badpvp.groups.allies, Legacy.NDB.db[name].city) and not badpvp.groups.overrideAllies then
            -- Don't target those from an allied city unless overrideAllies is true
            cecho("\n<firebrick>COMBAT NOTICE: Ignoring target call for " .. name .. " as they are from an allied city.")
            return false
        else
            return true
        end
    else
        cecho("\n<firebrick>COMBAT NOTICE: Ignoring target call for " .. name .. " as they don't exist in NDB.")
        return false
    end
end

function badpvp.verifyCaller(name)
    return table.contains(badpvp.groups.leaders, name)
end

function badpvp.processTargetList(numberOfTargets, listOfTargets)
    local ourTable = string.split(string.trim(listOfTargets), ", ")
    if numberOfTargets ~= tostring(#ourTable) then
        cecho("\n<firebrick>COMBAT NOTICE: Ignoring target list as it has more targets than expected")
        return
    end

    send("unenemy all")

    badpvp.groups.targetList = {}
    badpvp.groups.deadTargets = {}
    badpvp.groups.targetIndex = 1

    for k, name in pairs(ourTable) do
        if badpvp.verifyTarget(name) then
            table.insert(badpvp.groups.targetList, name)
            send("enemy " .. name)
        end
    end

    expandAlias("st " .. badpvp.groups.targetList[1])

    cecho("\n<firebrick>COMBAT NOTICE: Targetlist now set to " ..
    tostring(#badpvp.groups.targetList) .. " targets: " .. table.concat(badpvp.groups.targetList, ", "))
end

function badpvp.generateHelp()
    local tablemaker = require("BadPVP.MDK.ftext").TableMaker
    local formatter = require ("BadPVP.MDK.ftext").TextFormatter

    headerFormat = formatter:new({
        width = 80,
        cap = "<|",
        capColor = "<DarkSlateBlue>",
        textColor = "<white>",
        spacerColor = "<steel_blue>",
        inside = false,
        spacer = " ",
        alignment = "center",
        mirror = true
    })

    helpTable = tablemaker:new({
        title = "BadPVP Help",
        printTitle = false,
        printHeaders = true,
        frameColor = "<steel_blue>",
        separateRows = false,
        forceHeaderSeparator = true
    })
    helpTable:addColumn({
        name = "Command",
        width = 30,
        textColor = "<DarkSlateBlue>"
    })
    helpTable:addColumn({
        name = "Description",
        width = 50,
        textColor = "<DarkSlateGrey>"
    })
    --pvp
    helpTable:addRow({
        "pvp",
        "Generate your next pvp action."
    })
    --pvpnt
    helpTable:addRow({
        "<LightSlateBlue>pvpnt",
        "<LightSlateGrey>Manually move to the next target in the list."
    })
    --pvpreset
    helpTable:addRow({
        "pvpreset",
        "Reload the Engine and pick up new changes."
    })
    --pvpleaders (-|+)name
    helpTable:addRow({
        "<LightSlateBlue>pvpLeaders (-|+)name",
        "<LightSlateGrey>Add(+) or Remove(-) a group Leader"
    })
    --pvpleaders
    helpTable:addRow({
        "pvpleaders",
        "List all current group Leaders."
    })
    --pvphelp
    helpTable:addRow({
        "<LightSlateBlue>pvphelp",
        "<LightSlateGrey>Display this menu."
    })

    cecho(headerFormat:format("BadPVP Help Docs"))
    cecho(helpTable:assemble())

end